/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 0 references 
 * 
 * This implementation performs invocations of the getProperty() method of the ComponentContext API and
 * checks that the results conform to a series of specification requirements
 */
@Service(Service1.class)
public class Service1CCgetPropertyImpl implements Service1 {
	
	private ComponentContext componentContext;
	private String responseAccumulator = "";
	
	@Property
	public String serviceName = "service1";
	@Property
	public float floatProperty;		
	@Property(required=false)
	public int optionalProperty;

	public Service1CCgetPropertyImpl() {
		super();
	}
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		responseAccumulator += serviceName + " operation1 invoked getProperty";
		
		if( componentContext == null ) return responseAccumulator + " injected componentContext null";
		
		// JCA-TA-9033 - IllegalArgumentException thrown if named property does not have the type specified
		try {
		    componentContext.getProperty(java.util.HashSet.class, "floatProperty");
		    responseAccumulator += " did not get IllegalArgumentException for invalid property type";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid property type";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid property type: " + e.toString();
		} // end try	
		
		// JCA-TA-9032 - IllegalArgumentException thrown if component does not have the named property
		try {
		    componentContext.getProperty(Service1.class, "randomProp");
		    responseAccumulator += " did not get IllegalArgumentException invalid property name";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid property name";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid property name: " + e.toString();
		} // end try
		
		// JCA-TA-9031 - returns null for non-required property with no value set in the component configuration
		try {
		    Integer intValue = componentContext.getProperty(Integer.class, "optionalProperty");
		    if( intValue == null ) {
			    responseAccumulator += " expected null value for unconfigured optional property";
		    } else {
		    	responseAccumulator += " unexpected non-null value for unconfigured optional property";
		    }
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for value for unconfigured optional property";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for value for unconfigured optional property: " + e.toString();
		} // end try
		
		// JCA-TA-9030 - returns a valid property value for a named property of the component
		try {
		    Float floatValue = componentContext.getProperty(Float.class, "floatProperty");
		    if( floatValue == null ) {
			    responseAccumulator += " unexpected null property value for required property";
		    } else {
		    	responseAccumulator += " expected non-null property value for required property: " + floatValue.toString();
		    }
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for required property";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for required property: " + e.toString();
		} // end try		

		return responseAccumulator;
	} // end operation1

} // end class
