/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import java.util.concurrent.atomic.AtomicInteger;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.ParallelService;


/**
 * Java component implementation used to check whether an individual COMPOSITE instance
 * gets invoked by more than one thread at a time
 * 1 service with interface ParallelService
 * 0 references
 *
 * @author MikeEdwards
 *
 */
@Service(ParallelService.class)
@Scope("COMPOSITE")
public class parallelCompositeServiceImpl implements ParallelService {
	
	@Property
	public String serviceName = "service1";
	@Property
	public int waitTime = 1000;					// Wait time in milliseconds, default = 1 sec
	
	// A check value that will operate correctly if used by multiple threads
	AtomicInteger checkValue = new AtomicInteger();

	public int operation1(String input) {
		// Increment the checkValue
		checkValue.incrementAndGet();
		
		// wait for the prescribed time - this gives the chance for other threads to invoke this
		// service (should be on other threads)
		try {
			Object waiter = new Object();
			synchronized( waiter ) {
				waiter.wait(waitTime);
			} // end synchronized
		} catch (InterruptedException e) {
		} // end try
		
		int value = checkValue.getAndDecrement();
		// value should be the number of threads using this instance at the point
		// immediately before the getAndDecrement operation
		return value;
	} // end method operation1

}
