/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import java.util.concurrent.atomic.AtomicInteger;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.ParallelService;


/**
 * Java component implementation used to check whether an individual stateless instance
 * ever gets invoked by more than one thread at a time
 * 1 service with interface ParallelService
 * 0 references
 *
 * @author MikeEdwards
 *
 */
@Service(ParallelService.class)
public class parallelServiceImpl implements ParallelService {
	
	@Property
	public String serviceName = "service1";
	@Property
	public int waitTime = 5000;					// Wait time in milliseconds, default = 5 sec
	
	// A check value that will operate correctly if used by multiple threads
	AtomicInteger checkValue = new AtomicInteger();

	public int operation1(String input) {
		// Increment the checkValue
		checkValue.incrementAndGet();
		
		// wait for the prescribed time - this gives the chance for other threads to invoke this
		// service (but should be on other instances, not this one...)
		try {
			Object waiter = new Object();
			synchronized( waiter ) {
				waiter.wait(waitTime);
			} // end synchronized
		} catch (InterruptedException e) {
		} // end try
		
		int value = checkValue.get();
		// value should be 1 if no other thread has incremented the checkValue
		if( value == 1) return 0;
		return 1;
	} // end method operation1

}
