/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation
 * 1 service with interface Service1
 * 0 references
 *
 * The implementation is COMPOSITE scope
 * - it holds state information from one invocation of Service1.operation1 to the next invocation
 * - the input String from one invocation is returned in the result of the next invocation
 * 
 * @author MikeEdwards
 *
 */
@Service(Service1.class)
@Scope("COMPOSITE")
public class service1CompositeImpl implements Service1 {
	
	// This is a piece of state data, which depends on the implementation being COMPOSITE scope
	// so that all operations are against a single instance of this class, so that the state is
	// shared by all invocations.  The idea is that the state data is returned in the response to 
	// each invocation - and that the state is also updated by the input data in each invocation
	String state = "Initial";
	
	@Property
	public String serviceName = "service1";

	public String operation1(String input) {
		// Prepare response including the current value of the state
		String response = serviceName + " operation1 " + state;
		
		// Update state
		state = input;
		
		return response;
	} // end method operation1

} // end class service1CompositeImpl
