/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 1 reference (1..n) with interface Service1
 * all configured wires get called when service1 operation1 is invoked
 * 
 * This implementation acts as a coordinator between each of the invocations
 * - it sends different data as input to each reference invocation
 * - it records all the output data from all the invocations
 *
 */
@Service(Service1.class)
public class service1CoordinatorImpl implements Service1 {
	
	@Property
	public String serviceName = "service1";
	// Required = true + an array -> multiplicity 1..n
	@Reference(required=true)
	public Service1[] reference1 = null;

	public String operation1(String input) {
		String result = "";
		
		// Deal with the case where the reference has no targets supplied...
		// Should this throw an exception?
		if( reference1.length == 0 ) {
			return serviceName + " error: 1..n multiplicity reference 'reference1' has no targets";
		} // end if
		
		// Call each of the references in the array, concatenating the results
		for( int i=0 ; i < reference1.length; i++ ) {
			result = result + " ";
			result = result.concat( reference1[i].operation1(Integer.toString(i+1)) );
		} // end for
		return serviceName + " operation1 invoked" + result;
	}

}
