/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.RequestContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1Superset;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 0 references 
 * 
 * This implementation invokes the getServiceReference() method of the RequestContext API and
 * checks that the returned ServiceReference represents the Service offered by this component
 */
@Service(Service1Superset.class)
public class service1RequestContextImpl2 implements Service1Superset {
	
	private ComponentContext componentContext;
	
	@Property
	public String serviceName = "service1";
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		String result = "";
		String response = serviceName + " operation1 invoked";
		
		RequestContext theReqContext;
		try {
			theReqContext = componentContext.getRequestContext();
			if( theReqContext == null ) return response + " null RequestContext received from getRequestContext()";
		} catch (Exception e) {
			return response + " exception received from getRequestContext: " + e.toString();
		} // end try
		
		// Obtain the ServiceReference from the RequestContext
		ServiceReference<?> theServRef = theReqContext.getServiceReference();
		if( theServRef == null ) return response + " getServiceReference returned a null ServiceReference";
		
		// Get the business interface from the ServiceReference and check it is the right interface
		Class<?> theBusinessInterface = theServRef.getBusinessInterface();
		if( !(theBusinessInterface == Service1Superset.class) ) {
			return response + " getServiceReference returned incorrect business interface: " + theBusinessInterface.toString();
		} // end if
		
		// Get a service reference that can be invoked...
		Service1Superset referenceService = (Service1Superset) theServRef.getService();
		
		// Call a service operation on the service reference - it should actually call THIS same component
		result += "ServiceReference obtained: ";
		result += referenceService.operation2("checkService");

		return  response + " " + result;
	} // end operation1

	/**
	 * This method is used to check that the returned ServiceReference does indeed point to THIS
	 * service implementation
	 */
	public String operation2(String input) {
		
		return serviceName + " " + input + " operation2 invoked";
	} // end operation2

}
