/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 0 references 
 * 
 * This implementation performs invocations of the getService() method of the ComponentContext API and
 * checks that the results conform to a series of specification requirements
 */
@Service(Service1.class)
public class Service1CCgetServiceImpl implements Service1 {
	
	private ComponentContext componentContext;
	private String responseAccumulator = "";
	
	@Property
	public String serviceName = "service1";
	@Reference
	public Service1 requiredRef;		// 1..1 multiplicity
	@Reference(required=false)
	public Service1 singleRef;			// 0..1 multiplicity - *SHOULD NOT BE WIRED*
	@Reference(required=false)
	public Service1[] multiRef;			// 0..n multiplicity

	public Service1CCgetServiceImpl() {
		super();
	}
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		responseAccumulator += serviceName + " operation1 invoked getService";
		
		if( componentContext == null ) return responseAccumulator + " injected componentContext null";
		
		// JCA-TA-9001 - IllegalArgumentException thrown if called for an x..n reference
		try {
		    componentContext.getService(Service1.class, "multiRef");
		    responseAccumulator += " did not get IllegalArgumentException for 0..n ref";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for 0..n ref";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for 0..n ref: " + e.toString();
		} // end try
		
		// JCA-TA-9013 - IllegalArgumentException thrown if named reference does not have the businessInterface specified
		try {
		    componentContext.getService(Service2.class, "requiredRef");
		    responseAccumulator += " did not get IllegalArgumentException for invalid businessInterface";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid businessInterface";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid businessInterface: " + e.toString();
		} // end try	
		
		// JCA-TA-9012 - IllegalArgumentException thrown if component does not have the named reference
		try {
		    componentContext.getService(Service1.class, "randomRef");
		    responseAccumulator += " did not get IllegalArgumentException invalid reference name";
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " expected IllegalArgumentException received for invalid reference name";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for invalid reference name: " + e.toString();
		} // end try
		
		// JCA-TA-9011 - returns null for a 0..1 reference that is not wired
		try {
		    Service1 serviceRef = componentContext.getService(Service1.class, "singleRef");
		    if( serviceRef == null ) {
			    responseAccumulator += " expected null Service for unwired 0..1 reference";
		    } else {
		    	responseAccumulator += " unexpected non-null Service for unwired 0..1 reference";
		    }
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for unwired 0..1 reference";	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for unwired 0..1 reference: " + e.toString();
		} // end try
		
		// JCA-TA-9010 - returns a valid ServiceReference for a valid reference which is wired to a target service
		try {
		    Service1 serviceRef = componentContext.getService(Service1.class, "requiredRef");
		    if( serviceRef == null ) {
			    responseAccumulator += " unexpected null Service for wired 1..1 reference";
		    } else {
		    	responseAccumulator += " expected non-null Service for wired 1..1 reference";
		    	// Actually invoke the target service to verify that the returned ServiceReference is indeed a proxy for that service
		    	responseAccumulator += " " + serviceRef.operation1( "dynamic" );
		    }
		} catch ( IllegalArgumentException e ) {
			responseAccumulator += " unexpected IllegalArgumentException received for wired 1..1 reference";
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for wired 1..1 reference: " + e.toString();
		} // end try		

		return responseAccumulator;
	} // end operation1

} // end class
