/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 0 references 
 * 
 * This implementation performs invocations of the getURI() method of the ComponentContext API and
 * checks that the results conform to a series of specification requirements
 */
@Service(Service1.class)
public class Service1CCgetURIImpl implements Service1 {
	
	private ComponentContext componentContext;
	private String responseAccumulator = "";
	
	@Property
	public String serviceName = "service1";

	public Service1CCgetURIImpl() {
		super();
	}
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		responseAccumulator += serviceName + " operation1 invoked getURI";
		
		if( componentContext == null ) return responseAccumulator + " injected componentContext null";
		
		// JCA-TA-9009 - Returns the absolute URI of the component in the Domain
		try {
		    String componentURI = componentContext.getURI();
		    responseAccumulator += " getURI() returned URI = " + componentURI;	
		} catch ( Exception e ) {
			responseAccumulator += " other unexpected exception received for getURI: " + e.toString();
		} // end try

		return responseAccumulator;
	} // end operation1

} // end class
