/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import java.net.URI;
import java.util.Properties;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.client.SCAClientFactory;
import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.NoSuchDomainException;
import org.oasisopen.sca.NoSuchServiceException;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 1 reference "reference1" with interface Service1 
 * 
 * This implementation performs invocations of the SCAClient API and
 * checks that the results conform to a series of specification requirements
 */
@Service(Service1.class)
public class Service1SCAClientImpl implements Service1 {
	
	private String responseAccumulator = "";
	private URI validDomainURI;
	private URI invalidDomainURI;
	
	@Property
	public String serviceName = "service1";
	@Reference(required=false)
	public Service1 reference1;
	@Property
	public String validDomainURIString;
	@Property
	public String invalidDomainURIString;
	@Property(required=false)
	public String validServiceName = "TEST_JCA_9016Component2/Service1";
	@Property(required=false)
	public String invalidServiceName = "FooComponent/BarService";

	public Service1SCAClientImpl() {
		super();
	} // end constructor
	
	@Init
	public void initialize() {
		// Set up the DomainURIs to use from the values of the related Properties
		try {
			validDomainURI = new URI( validDomainURIString );
			invalidDomainURI = new URI( invalidDomainURIString );
		} catch (Exception e) {
			System.out.println("Received exception creating Domain URIs: " + e.toString());
		}		
	} // end method initialize()

	public String operation1(String input) {
		responseAccumulator += serviceName + " operation1 invoked SCAClient";
		
		// JCA-TA-9051 - SCAClientFactory.newInstance(URI) returns a valid client factory for a valid DomainURI
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(validDomainURI);
		    if( clientFactory == null ) {
			    responseAccumulator += " unexpected null client factory";
		    } else {
		    	responseAccumulator += " expected non-null client factory";
		    }
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for valid URI: " + e.toString();
		} // end try		
		
		// JCA-TA-9052 - SCAClientFactory.newInstance(URI) throws a NoSuchDomainException for an invalid DomainURI
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(invalidDomainURI);
		    if( clientFactory == null ) {
			    responseAccumulator += " unexpected null client factory";
		    } else {
		    	responseAccumulator += " unexpected non-null client factory";
		    }
		} catch ( NoSuchDomainException e ) {
			responseAccumulator += " expected exception received for invalid URI: NoSuchDomainException";
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for invalid URI: " + e.toString();
		} // end try	

		// Simple Properties object for subsequent invocations
		Properties theProperties = new Properties();
		theProperties.put("aKey", "aValue");
		
		// JCA-TA-9053 - SCAClientFactory.newInstance(Properties, URI) returns a valid client factory for a valid DomainURI
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(theProperties, validDomainURI);
		    if( clientFactory == null ) {
			    responseAccumulator += " unexpected null client factory";
		    } else {
		    	responseAccumulator += " expected non-null client factory";
		    }
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for valid URI: " + e.toString();
		} // end try		
		
		// JCA-TA-9054 - SCAClientFactory.newInstance(Properties, URI) throws a NoSuchDomainException for an invalid DomainURI
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(theProperties, invalidDomainURI);
		    if( clientFactory == null ) {
			    responseAccumulator += " unexpected null client factory";
		    } else {
		    	responseAccumulator += " unexpected non-null client factory";
		    }
		} catch ( NoSuchDomainException e ) {
			responseAccumulator += " expected exception received for invalid URI: NoSuchDomainException";
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for invalid URI: " + e.toString();
		} // end try
		
		// Classloader for use in subsequent invocations
		ClassLoader theClassloader = new TestClassLoader();

		// JCA-TA-9055 - SCAClientFactory.newInstance(Classloader, URI) returns a valid client factory for a valid DomainURI
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(theClassloader, validDomainURI);
		    if( clientFactory == null ) {
			    responseAccumulator += " unexpected null client factory";
		    } else {
		    	responseAccumulator += " expected non-null client factory";
		    }
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for valid URI: " + e.toString();
		} // end try		
		
		// JCA-TA-9056 - SCAClientFactory.newInstance(Classloader, URI) throws a NoSuchDomainException for an invalid DomainURI
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(theClassloader, invalidDomainURI);
		    if( clientFactory == null ) {
			    responseAccumulator += " unexpected null client factory";
		    } else {
		    	responseAccumulator += " unexpected non-null client factory";
		    }
		} catch ( NoSuchDomainException e ) {
			responseAccumulator += " expected exception received for invalid URI: NoSuchDomainException";
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for invalid URI: " + e.toString();
		} // end try	
		
		// JCA-TA-9057 - SCAClientFactory.newInstance(Properties, Classloader, URI) returns a valid client factory for a valid DomainURI
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(theProperties, theClassloader, validDomainURI);
		    if( clientFactory == null ) {
			    responseAccumulator += " unexpected null client factory";
		    } else {
		    	responseAccumulator += " expected non-null client factory";
		    }
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for valid URI: " + e.toString();
		} // end try		
		
		// JCA-TA-9058 - SCAClientFactory.newInstance(Properties, Classloader, URI) throws a NoSuchDomainException for an invalid DomainURI
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(theProperties, theClassloader, invalidDomainURI);
		    if( clientFactory == null ) {
			    responseAccumulator += " unexpected null client factory";
		    } else {
		    	responseAccumulator += " unexpected non-null client factory";
		    }
		} catch ( NoSuchDomainException e ) {
			responseAccumulator += " expected exception received for invalid URI: NoSuchDomainException";
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for invalid URI: " + e.toString();
		} // end try
		
		// JCA-TA-9059 - SCAClientFactory.getService method returns a proxy object for the target component service when
		//               invoked with a valid serviceName and valid business interface
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(validDomainURI);
		    if( clientFactory != null ) {
		    	Service1 referenceProxy = clientFactory.getService(Service1.class, validServiceName);
		    	if( referenceProxy == null ) {
		    		responseAccumulator += " unexpected null reference proxy";
		        } else {
		    	    responseAccumulator += " expected non-null reference proxy";
		    	    // Invoke the proxy and ensure that the correct service is invoked
		    	    try {
		    	    	responseAccumulator += referenceProxy.operation1("dynamic invocation");
		    	    } catch (Throwable t ) {
		    	    	responseAccumulator += " unexpected throwable received for getService(): " + t.toString();
		    	    }
		        } // end if
		    } // end if
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for getService(): " + e.toString();
		} // end try
	
		// JCA-TA-9060 - SCAClientFactory.getService method throws NoSuchServiceException exception when
		//               the service identified by serviceName does not exist
		try {
		    SCAClientFactory clientFactory = SCAClientFactory.newInstance(validDomainURI);
		    if( clientFactory != null ) {
		    	Service1 referenceProxy = clientFactory.getService(Service1.class, invalidServiceName);
		    	if( referenceProxy == null ) {
		    		responseAccumulator += " unexpected null reference proxy";
		        } else {
		    	    responseAccumulator += " unexpected non-null reference proxy";
		        } // end if
		    } // end if
		} catch ( NoSuchServiceException e ) {
			responseAccumulator += " expected NoSuchServiceException exception received for getService()";
		} catch ( Exception e ) {
			responseAccumulator += " unexpected exception received for getService(): " + e.toString();
		} // end try
	
		return responseAccumulator;
	} // end method operation1
	
	
	
	public class TestClassLoader extends ClassLoader {
		
		public TestClassLoader() {
			super( Thread.currentThread().getContextClassLoader() );
		} // end constructor
		
	} // end class TestClassLoader

} // end class
