/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *     
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.Service1;


/**
 * Java component implementation with
 * 1 service with interface Service1
 * 1 reference "reference1" (1..1) with interface Service1
 * 1 reference "dataStore" (1..1) with interface DataStore
 * 
 * This implementation acts as a controller for a test on the component implementation
 * which is the target of reference1 which is an async service implementation
 * - reference1 is invoked 
 * - when the reference1 invocation has completed, the dataStore is queried for the result data
 *   which has been stored in the data store by the lifecycle component implementation
 */
@Service(Service1.class)
public class asyncControllerImpl implements Service1 {
	
	@Property
	public String serviceName = "service1";
	@Property
	public String storeName	= null;
	// Required = true -> multiplicity 1..1
	@Reference(required=true)
	public Service1 reference1 	= null;
	@Reference(required=true)
	public DataStore dataStore 	= null;

	public String operation1(String input) {
		String result = "";
		
		// Invoke the target asynchronous service implementation
		// This MAY generate an exception for some of the tests - absorb this here since
		// the "real result" is placed into the data store and is retrieved from there irrespective of the exception
		try {
			result += reference1.operation1( input );
		} catch (Exception e ) {
			e.printStackTrace();
			result += "Invocation of async service implemenatation threw exception: " + e.getClass().getName() + e.getMessage();
		} // end try
		
		// Wait here for the async service to complete...	
		try {
			Thread.sleep(2000);
		} catch (InterruptedException e) {
			// Nothing to do here...
		}
		
		// Retrieve result data from the data store
		result += dataStore.retrieveData( storeName );
		
		return result;
	}

} // end class asyncControllerImpl
