/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import org.oasisopen.sca.ComponentContext;
import org.oasisopen.sca.RequestContext;
import org.oasisopen.sca.ServiceReference;
import org.oasisopen.sca.annotation.Context;
import org.oasisopen.sca.annotation.Property;
import org.oasisopen.sca.annotation.Reference;
import org.oasisopen.sca.annotation.Service;
import org.oasisopen.sca.test.Service1;

/**
 * Java component implementation with
 * 1 service with interface Service1,
 * 1 reference (1..1) with interface Service3WithCallback which has a Callback
 * interface Service3Callback which is implemented by this class
 * reference gets called when service1 operation1 is invoked
 * 
 * The callback operation's method accesses the getRequestContext() method of the 
 * ComponentContext API during the processing of a callback method and checks that
 * returned RequestContext is not null
 */
@Service(Service1.class)
public class service1CallbackContextImpl2 implements Service1, Service3Callback {

	private ComponentContext componentContext;
	
	@Property
	public String serviceName = "service1";
	// Reference with an implied multiplicity of (1..1) since required=true is the default
	@Reference
	public Service3WithCallback reference1 = null;
	
	/**
	 * Setter for the ComponentContext used by this implementation
	 * @param theContext - the ComponentContext
	 */
	@Context
	public void setContext( ComponentContext theContext ) {
		componentContext = theContext;
	} // end method setContext

	public String operation1(String input) {
		String result = reference1.operation3(input);
		return serviceName + " operation1 invoked" + " " + result;
	}
	
	public String callback1( String input ) {
		String result = "";
		String response = serviceName + " callback1 invoked";
		
		try {
			RequestContext theReqContext = componentContext.getRequestContext();
			if( theReqContext == null ) return response + " null RequestContext received from getRequestContext()";
			result = "RequestContext - serviceName: " + theReqContext.getServiceName();
		} catch (Exception e) {
			return response + " exception received from getRequestContext: " + e.toString();
		} // end try

		return  response + " " + result;
	} // end operation1
	
}
