/*
 *
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 *   
 */
package org.oasisopen.sca.test;

import java.util.concurrent.CountDownLatch;
import java.util.concurrent.TimeUnit;

import org.oasisopen.sca.annotation.*;
import org.oasisopen.sca.test.ServiceOneWay;


/**
 * Java component implementation
 * 1 service with interface ServiceOneWay
 * 0 references
 *
 * The implementation is COMPOSITE scope and is stateful
 * - it remembers whether its OneWay method inonly1 was called
 * - it reports this when its request/Response method is called
 */
@Service(ServiceOneWay.class)
@Scope("COMPOSITE")
public class serviceOneWayImpl2 implements ServiceOneWay {
	
	String status = "OneWay method inonly not called";
	
	CountDownLatch firstLatch = new CountDownLatch( 1 );
	CountDownLatch secondLatch = new CountDownLatch( 1 );
	
	@Property(required=true)
	public String serviceName = "service1";
	
	/*
	 * Both methods use a latch to wait to complete until the other method starts.
	 */
	public String operation1(String input) {
		firstLatch.countDown();
		/*
		 * Wait until the OneWay method completes
		 */
		try {
			secondLatch.await( 1, TimeUnit.SECONDS );
		} catch (InterruptedException e) {
			// Do nothing
		}
		return serviceName + " operation1 invoked " + status;
	}

	@OneWay
	public void inonly1(String input) {
		boolean test = false;
		/*
		 * Wait until the Req/Rsp method is called
		 */
		try {
			test = firstLatch.await( 1, TimeUnit.SECONDS );
		} catch (InterruptedException e) {
			// Do nothing
		}
		/*
		 * If this method is blocking, the wait on firstLatch will time out
		 * and test will be false
		 */
		if (test)
			status = "OneWay method inonly previously called with " + input;
		secondLatch.countDown();
	} // end inonly1

} // end class serviceOneWayImpl2
